@@ -481,6 +481,10 @@ typedef enum {
  * @brief Default value for the #SPLT_OPT_PARAM_MIN_LENGTH option
  */
 #define SPLT_DEFAULT_PARAM_MINIMUM_LENGTH 0.0
+/**
+ * @brief Default value for the #SPLT_OPT_PARAM_MIN_TRACK_LENGTH option
+ */
+#define SPLT_DEFAULT_PARAM_MINIMUM_TRACK_LENGTH 0.0
 /**
  * @brief Default value for the #SPLT_OPT_PARAM_GAP option
  */
@@ -634,6 +638,12 @@ typedef struct {
     a valid splitpoint
   */
   float parameter_minimum_length;
+  /*! The minimum track length for option_silence_mode [in seconds]
+ 
+    A positive float of the minimum number of seconds to be considered
+    a valid splitpoint
+  */
+  float parameter_min_track_length;
 
   //!possible values are #splt_str_format
   int artist_tag_format;
@@ -1159,6 +1169,15 @@ typedef enum {
    * Default is #SPLT_DEFAULT_PARAM_MINIMUM_LENGTH
    */
   SPLT_OPT_PARAM_MIN_LENGTH,
+  /**
+   * Minimum number of seconds to be considered a valid track\n
+   * All tracks shorter than this value are discarded.
+   *
+   * The option can take positive float values.
+   *
+   * Default is #SPLT_DEFAULT_PARAM_MINIMUM_TRACK_LENGTH
+   */
+  SPLT_OPT_PARAM_MIN_TRACK_LENGTH,
   /**
    * Format of the artist tag from filename.
    * Possible values are #splt_str_format
@@ -1303,7 +1322,7 @@ static const char splt_id3v1_genres[SPLT_ID3V1_NUMBER_OF_GENRES][25] = {
 /**
  * @brief Package version
  */
-#define SPLT_PACKAGE_VERSION "0.7"
+#define SPLT_PACKAGE_VERSION "0.7.1"
 #endif
 /**
  * @brief Package authors
@@ -1578,6 +1597,7 @@ void mp3splt_set_oformat(splt_state *state,
 int mp3splt_count_silence_points(splt_state *state, int *error);
 
 int mp3splt_set_silence_points(splt_state *state, int *error);
+void mp3splt_set_trim_silence_points(splt_state *state, int *error);
 
 //returns the version of libmp3splt
 void mp3splt_get_version(char *version);
