@@ -124,6 +124,8 @@ static const unsigned long splt_mp3_crctab[256] = {
 static void splt_mp3_save_end_point(splt_state *state, splt_mp3_state *mp3state,
     int save_end_point, off_t end)
 {
+  mp3state->end_non_zero = end;
+
   if (save_end_point)
   {
     mp3state->end = end;
@@ -1111,6 +1113,7 @@ static splt_mp3_state *splt_mp3_info(FILE *file_input, splt_state *state,
   //we initialise default values
   mp3state->frames = 1;
   mp3state->end = 0;
+  mp3state->end_non_zero = 0;
   mp3state->first = 1;
   mp3state->file_input = file_input;
   mp3state->framemode = framemode;
@@ -1260,6 +1263,8 @@ static splt_mp3_state *splt_mp3_info(FILE *file_input, splt_state *state,
 
   //we put useful infos in the state
   mp3state->mp3file.firsth = (off_t) (mp3state->bytes - len);
+  splt_d_print_debug(state, "mp3 firsth bytes = %ld\n", mp3state->bytes);
+  splt_d_print_debug(state, "mp3 firsth len = %ld\n", len);
   mp3state->bytes = mp3state->mp3file.firsth;
   mp3state->headw = 
     (unsigned long) ((mp3state->data_ptr[0] << 24) | 
@@ -2159,6 +2164,12 @@ bloc_end:
 
         splt_c_put_progress_text(state,SPLT_PROGRESS_PREPARE);
 
+        //this happens when we end and start on the same frame
+        if (mp3state->frames >= fbegin && mp3state->end_non_zero != 0)
+        {
+          begin = mp3state->end_non_zero;
+        }
+
         // Finds begin by counting frames
         while (mp3state->frames < fbegin)
         {
@@ -3375,8 +3386,8 @@ void splt_pl_set_original_tags(splt_state *state, int *error)
 
 void splt_pl_clear_original_tags(splt_original_tags *original_tags)
 {
-  tag_bytes_and_size *bytes_and_size = 
-    (tag_bytes_and_size *) original_tags->all_original_tags;
+#ifndef NO_ID3TAG
+  tag_bytes_and_size *bytes_and_size = (tag_bytes_and_size *) original_tags->all_original_tags;
 
   if (!bytes_and_size) {
     return;
@@ -3392,6 +3403,7 @@ void splt_pl_clear_original_tags(splt_original_tags *original_tags)
 
   free(original_tags->all_original_tags);
   original_tags->all_original_tags = NULL;
+#endif
 }
 
 //@}
