@@ -44,7 +44,6 @@ The Plug-in that handles ogg vorbis files
 
 #include <time.h>
 #include <string.h>
-#include <math.h>
 #include <locale.h>
 
 #ifdef __WIN32__
@@ -56,6 +55,7 @@ The Plug-in that handles ogg vorbis files
 #include "ogg.h"
 #include "ogg_utils.h"
 #include "ogg_silence.h"
+#include "ogg_new_stream_handler.h"
 
 #define FIRST_GRANPOS 1
 
@@ -72,8 +72,8 @@ The Plug-in that handles ogg vorbis files
 /* some function prototypes */
 
 splt_ogg_state *splt_ogg_info(FILE *in, splt_state *state, int *error);
-static vorbis_comment *clone_vorbis_comment(vorbis_comment *comment);
-static void free_vorbis_comment(vorbis_comment *vc, short cloned_vorbis_comment);
+static void splt_ogg_clear_sync_in(splt_ogg_state *oggstate);
+static void splt_ogg_clear_sync_in_and_free(splt_ogg_state *oggstate);
 
 /****************************/
 /* ogg utils */
@@ -138,196 +138,70 @@ void splt_ogg_get_info(splt_state *state, FILE *file_input, int *error)
   }
 }
 
-static char *splt_ogg_trackstring(int number, int *error)
-{
-  char *track = NULL;
-
-  if (number > 0)
-  {
-    int len = 0, i;
-    len = ((int) (log10((double) (number)))) + 1;
-
-    if ((track = malloc(len + 1))==NULL)
-    {
-      *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-      return NULL;
-    }
-    memset(track, 0, len + 1);
-    for (i=len-1; i >= 0; i--)
-    {
-      track[i] = ((number%10) | 0x30);
-      number /= 10;
-    }
-  }
-
-  return track;
-}
-
-//saves a packet
-static splt_v_packet *splt_ogg_save_packet(ogg_packet *packet, int *error)
-{
-  splt_v_packet *p = NULL;
-
-  //if we have no header, we will have bytes < 0
-  p = malloc(sizeof(splt_v_packet));
-  if (!p)
-  {
-    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-    return p;
-  }
-
-  p->length = packet->bytes;
-  p->packet = malloc(p->length);
-  if (! p->packet)
-  {
-    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-    free(p);
-    p = NULL;
-    return p;
-  }
-  memcpy(p->packet, packet->packet, p->length);
-
-  return p;
-}
-
-//frees a packet
-static void splt_ogg_free_packet(splt_v_packet **p)
-{
-  if (p)
-  {
-    if (*p)
-    {
-      if((*p)->packet)
-      {
-        free((*p)->packet);
-        (*p)->packet = NULL;
-      }
-      free(*p);
-      *p = NULL;
-    }
-  }
-}
-
-/* Returns 0 for success, or -1 on failure. */
-static int splt_ogg_write_pages_to_file(splt_state *state, 
-    ogg_stream_state *stream, FILE *file, int flush, int *error,
-    const char *output_fname)
-{
-  ogg_page page;
-
-  if (flush)
-  {
-    while (ogg_stream_flush(stream, &page))
-    {
-      if (splt_io_fwrite(state, page.header, 1, page.header_len, file) < page.header_len)
-      {
-        goto write_error;
-      }
-      if (splt_io_fwrite(state, page.body, 1, page.body_len, file) < page.body_len)
-      {
-        goto write_error;
-      }
-    }
-  }
-  else
-  {
-    while (ogg_stream_pageout(stream, &page))
-    {
-      if (splt_io_fwrite(state, page.header,1,page.header_len, file) < page.header_len)
-      {
-        goto write_error;
-      }
-      if (splt_io_fwrite(state, page.body,1,page.body_len, file) < page.body_len)
-      {
-        goto write_error;
-      }
-    }
-  }
-
-  return 0;
-
-write_error:
-  splt_e_set_error_data(state, output_fname);
-  *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
-  return -1;
-}
-
-static void splt_ogg_submit_headers_to_stream(ogg_stream_state *stream, 
-    splt_ogg_state *oggstate)
-{
-  int i;
-  for(i=0;i<3;i++)
-  {
-    ogg_packet p;
-    p.bytes = oggstate->headers[i]->length;
-    p.packet = oggstate->headers[i]->packet;
-    p.b_o_s = ((i==0)?1:0);
-    p.e_o_s = 0;
-    p.granulepos=0;
-
-    ogg_stream_packetin(stream, &p);
-  }
-}
-
 /* Full cleanup of internal state and vorbis/ogg structures */
 static void splt_ogg_v_free(splt_ogg_state *oggstate)
 {
-  if(oggstate)
+  if (!oggstate)
   {
-    if(oggstate->packets)
-    {
-      splt_ogg_free_packet(&oggstate->packets[0]);
-      splt_ogg_free_packet(&oggstate->packets[1]);
-      free(oggstate->packets);
-      oggstate->packets = NULL;
-    }
-    if(oggstate->headers)
-    {
-      int i;
-      for(i=0; i < 3; i++)
-      {
-        splt_ogg_free_packet(&oggstate->headers[i]);
-      }
-      free(oggstate->headers);
-      oggstate->headers = NULL;
-    }
+    return;
+  }
 
-    free_vorbis_comment(&oggstate->vc, oggstate->cloned_vorbis_comment);
-    oggstate->cloned_vorbis_comment = 2;
+  if(oggstate->packets)
+  {
+    splt_ogg_free_packet(&oggstate->packets[0]);
+    splt_ogg_free_packet(&oggstate->packets[1]);
+    free(oggstate->packets);
+    oggstate->packets = NULL;
+  }
 
-    if(oggstate->vb)
-    {
-      vorbis_block_clear(oggstate->vb);
-      free(oggstate->vb);
-      oggstate->vb = NULL;
-    }
-    if(oggstate->vd)
-    {
-      vorbis_dsp_clear(oggstate->vd);
-      free(oggstate->vd);
-      oggstate->vd = NULL;
-    }
-    //only free the input if different from stdin
-    if(oggstate->stream_in && oggstate->in != stdin)
-    {
-      ogg_stream_clear(oggstate->stream_in);
-      free(oggstate->stream_in);
-      oggstate->stream_in = NULL;
-    }
-    if(oggstate->sync_in)
-    {
-      ogg_sync_clear(oggstate->sync_in);
-      free(oggstate->sync_in);
-      oggstate->sync_in = NULL;
-    }
-    if (oggstate->vi)
-    {
-      vorbis_info_clear(oggstate->vi);
-      free(oggstate->vi);
-      oggstate->vi = NULL;
-    }
-    free(oggstate);
-    oggstate = NULL;
+  splt_ogg_free_oggstate_headers(oggstate);
+
+  splt_ogg_free_vorbis_comment(&oggstate->vc, oggstate->cloned_vorbis_comment);
+  oggstate->cloned_vorbis_comment = 2;
+
+  if(oggstate->vb)
+  {
+    vorbis_block_clear(oggstate->vb);
+    free(oggstate->vb);
+    oggstate->vb = NULL;
+  }
+
+  if(oggstate->vd)
+  {
+    vorbis_dsp_clear(oggstate->vd);
+    free(oggstate->vd);
+    oggstate->vd = NULL;
+  }
+
+  //only free the input if different from stdin
+  if(oggstate->stream_in && oggstate->in != stdin)
+  {
+    ogg_stream_clear(oggstate->stream_in);
+    free(oggstate->stream_in);
+    oggstate->stream_in = NULL;
+  }
+
+  splt_ogg_clear_sync_in_and_free(oggstate);
+
+  if (oggstate->vi)
+  {
+    vorbis_info_clear(oggstate->vi);
+    free(oggstate->vi);
+    oggstate->vi = NULL;
+  }
+
+  free(oggstate);
+  oggstate = NULL;
+}
+
+static void splt_ogg_clear_sync_in_and_free(splt_ogg_state *oggstate)
+{
+  if (oggstate->sync_in)
+  {
+    ogg_sync_clear(oggstate->sync_in);
+
+    free(oggstate->sync_in);
+    oggstate->sync_in = NULL;
   }
 }
 
@@ -360,16 +234,18 @@ static splt_ogg_state *splt_ogg_v_new(int *error)
   {
     goto error;
   }
-  if ((oggstate->headers = malloc(sizeof(splt_v_packet)*3))==NULL)
+
+  if ((oggstate->headers = malloc(sizeof(splt_v_packet) * TOTAL_HEADER_PACKETS))==NULL)
   {
     goto error;
   }
-  memset(oggstate->headers, 0, sizeof(splt_v_packet)*3);
-  if ((oggstate->packets = malloc(sizeof(splt_v_packet)*2))==NULL)
+  memset(oggstate->headers, 0, sizeof(splt_v_packet) * TOTAL_HEADER_PACKETS);
+
+  if ((oggstate->packets = malloc(sizeof(splt_v_packet) * 2))==NULL)
   {
     goto error;
   }
-  memset(oggstate->packets, 0, sizeof(splt_v_packet)*2);
+  memset(oggstate->packets, 0, sizeof(splt_v_packet) * 2);
 
   oggstate->cloned_vorbis_comment = 2;
 
@@ -397,161 +273,6 @@ void splt_ogg_state_free(splt_state *state)
 /****************************/
 /* ogg tags */
 
-static void add_tag_and_equal(const char *tag_name, const char *value, splt_array *to_delete, 
-    int *error)
-{
-  if (value == NULL) { return; }
-
-  size_t size = strlen(tag_name) + 2;
-  char *tag_and_equal = malloc(size);
-  if (tag_and_equal == NULL)
-  {
-    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-    return;
-  }
-  snprintf(tag_and_equal, size, "%s=", tag_name);
-
-  int err = splt_array_append(to_delete, tag_and_equal);
-  if (err == -1)
-  {
-    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-  }
-}
-
-static splt_array *build_tag_and_equal_to_delete(const char *artist, 
-    const char *album, const char *title, const char *tracknum,
-    const char *date, const char *genre, const char *comment, int *error)
-{
-  splt_array *to_delete = splt_array_new();
-
-  add_tag_and_equal(SPLT_OGG_TITLE, title, to_delete, error);
-  if (*error < 0) { goto error; }
-  add_tag_and_equal(SPLT_OGG_ARTIST, artist, to_delete, error);
-  if (*error < 0) { goto error; }
-  add_tag_and_equal(SPLT_OGG_ALBUM, album, to_delete, error);
-  if (*error < 0) { goto error; }
-  if (date != NULL && strlen(date) > 0) { add_tag_and_equal(SPLT_OGG_DATE, date, to_delete, error); }
-  if (*error < 0) { goto error; }
-  add_tag_and_equal(SPLT_OGG_GENRE, genre, to_delete, error);
-  if (*error < 0) { goto error; }
-  add_tag_and_equal(SPLT_OGG_TRACKNUMBER, tracknum, to_delete, error);
-  if (*error < 0) { goto error; }
-  add_tag_and_equal(SPLT_OGG_COMMENT, comment, to_delete, error);
-  if (*error < 0) { goto error; }
-
-  return to_delete;
-
-error:
-  splt_array_free(&to_delete);
-  return NULL;
-}
-
-static void delete_all_non_null_tags(vorbis_comment *vc, 
-    const char *artist, const char *album, const char *title,
-    const char *tracknum, const char *date, const char *genre, 
-    const char *comment, int *error)
-{
-  char *vendor_backup = NULL;
-  splt_array *tag_and_equal_to_delete = NULL;
-  splt_array *comments = NULL;
-  long i = 0, j = 0;
-
-  tag_and_equal_to_delete = 
-    build_tag_and_equal_to_delete(artist, album, title, tracknum, date, genre, comment, error);
-  if (*error < 0) { return; }
-
-  comments = splt_array_new();
-  if (comments == NULL) { goto end; }
-
-  for (i = 0;i < vc->comments; i++)
-  {
-    short keep_comment = SPLT_TRUE;
-
-    long number_of_tags_to_delete = splt_array_get_number_of_elements(tag_and_equal_to_delete);
-    for (j = 0;j < number_of_tags_to_delete; j++)
-    {
-      char *tag_and_equal = (char *) splt_array_get(tag_and_equal_to_delete, j);
-
-      if (strncasecmp(vc->user_comments[i], 
-            tag_and_equal, strlen(tag_and_equal)) == 0)
-      {
-        keep_comment = SPLT_FALSE;
-        break;
-      }
-    }
-
-    if (keep_comment)
-    {
-      char *user_comment = NULL;
-      int err = splt_su_append(&user_comment, 
-          vc->user_comments[i], vc->comment_lengths[i], NULL);
-      if (err <  0) { *error = err; goto end; }
-      err = splt_array_append(comments, user_comment);
-      if (err == -1) { *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY; goto end; }
-    }
-  }
-
-  int err = splt_su_copy(vc->vendor, &vendor_backup);
-  if (err < 0) { *error = err; goto end; }
-
-  vorbis_comment_clear(vc);
-
-  long tags_to_keep = splt_array_get_number_of_elements(comments);
-  for (i = 0;i < tags_to_keep; i++)
-  {
-    char *user_comment = splt_array_get(comments, i);
-    vorbis_comment_add(vc, user_comment);
-    free(user_comment);
-    user_comment = NULL;
-  }
-
-  if (vendor_backup)
-  {
-    splt_su_set(&vc->vendor, vendor_backup, strlen(vendor_backup), NULL);
-  }
-
-end:
-  if (vendor_backup)
-  {
-    free(vendor_backup);
-    vendor_backup = NULL;
-  }
-
-  splt_array_free(&comments);
-
-  long number_of_tags_to_delete = splt_array_get_number_of_elements(tag_and_equal_to_delete);
-  for (j = 0;j < number_of_tags_to_delete; j++)
-  {
-    char *tag_and_equal = (char *) splt_array_get(tag_and_equal_to_delete, j);
-    if (tag_and_equal) { free(tag_and_equal); }
-  }
-  splt_array_free(&tag_and_equal_to_delete);
-}
-
-//puts tags in vc
-//what happens if 'vorbis_comment_add_tag(..)' fails ?
-//- ask vorbis developers
-static void splt_ogg_v_comment(splt_state *state, vorbis_comment *vc, char *artist,
-    char *album, char *title, char *tracknum, char *date, char *genre, char *comment,
-    int *error)
-{
-  if (splt_o_get_int_option(state, SPLT_OPT_TAGS) == SPLT_TAGS_ORIGINAL_FILE &&
-      state->original_tags.tags.tags_version == 0)
-  {
-    return;
-  }
-
-  delete_all_non_null_tags(vc, artist, album, title, tracknum, date, genre, comment, error);
-
-  if (title != NULL) { vorbis_comment_add_tag(vc, SPLT_OGG_TITLE, title); }
-  if (artist != NULL) { vorbis_comment_add_tag(vc, SPLT_OGG_ARTIST, artist); }
-  if (album != NULL) { vorbis_comment_add_tag(vc, SPLT_OGG_ALBUM, album); }
-  if (date != NULL && strlen(date) > 0) { vorbis_comment_add_tag(vc, SPLT_OGG_DATE, date); }
-  if (genre != NULL) { vorbis_comment_add_tag(vc, SPLT_OGG_GENRE, genre); }
-  if (tracknum != NULL) { vorbis_comment_add_tag(vc, SPLT_OGG_TRACKNUMBER, tracknum); }
-  if (comment != NULL) { vorbis_comment_add_tag(vc, SPLT_OGG_COMMENT, comment); }
-}
-
 //macro used only in the following function splt_ogg_get_original_tags
 #define OGG_VERIFY_ERROR() \
 if (err != SPLT_OK) \
@@ -633,7 +354,7 @@ void splt_ogg_get_original_tags(const char *filename,
 
   splt_tu_set_original_tags_field(state, SPLT_TAGS_VERSION, &has_tags);
 
-  vorbis_comment *cloned_comment = clone_vorbis_comment(vc_local);
+  vorbis_comment *cloned_comment = splt_ogg_clone_vorbis_comment(vc_local);
   if (cloned_comment == NULL)
   {
     err = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
@@ -643,68 +364,12 @@ void splt_ogg_get_original_tags(const char *filename,
   splt_tu_set_original_tags_data(state, cloned_comment);
 }
 
-//puts the ogg tags
-void splt_ogg_put_tags(splt_state *state, int *error)
-{
-  splt_d_print_debug(state,"Setting ogg tags ...\n");
-
-  splt_ogg_state *oggstate = state->codec;
-
-  free_vorbis_comment(&oggstate->vc, oggstate->cloned_vorbis_comment);
-  oggstate->cloned_vorbis_comment = 2;
-
-  if (splt_o_get_int_option(state, SPLT_OPT_TAGS) == SPLT_NO_TAGS)
-  {
-    return;
-  }
-
-  splt_tags *tags = splt_tu_get_current_tags(state);
-  if (!tags)
-  {
-    return;
-  }
-
-  char *track_string = splt_ogg_trackstring(tags->track, error);
-  if (*error < 0) { return; }
-
-  char *artist_or_performer = splt_tu_get_artist_or_performer_ptr(tags);
-
-  vorbis_comment *original_vc = 
-    (vorbis_comment *) splt_tu_get_original_tags_data(state);
-
-  if (tags->set_original_tags && original_vc)
-  {
-    vorbis_comment *cloned_vc = clone_vorbis_comment(original_vc);
-    if (cloned_vc == NULL)
-    {
-      *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-      goto error;
-    }
-
-    oggstate->vc = *cloned_vc;
-    free(cloned_vc);
-    oggstate->cloned_vorbis_comment = SPLT_TRUE;
-  }
-  else {
-    vorbis_comment_init(&oggstate->vc);
-    oggstate->cloned_vorbis_comment = SPLT_FALSE;
-  }
-
-  splt_ogg_v_comment(state, &oggstate->vc,
-      artist_or_performer, tags->album, tags->title, track_string,
-      tags->year, tags->genre, tags->comment, error);
-
-error:
-  free(track_string);
-  track_string = NULL;
-}
-
 /****************************/
 /* ogg infos */
 
 //Pull out and save the 3 header packets from the input file.
 //-returns -1 if error and error is set in '*error'
-static int splt_ogg_process_headers(splt_ogg_state *oggstate, int *error)
+static int splt_ogg_read_headers_and_save_them(splt_state *state, splt_ogg_state *oggstate, int *error)
 {
   ogg_page page;
   ogg_packet packet;
@@ -713,7 +378,6 @@ static int splt_ogg_process_headers(splt_ogg_state *oggstate, int *error)
   char *buffer = NULL;
 
   ogg_sync_init(oggstate->sync_in);
-
   vorbis_info_init(oggstate->vi);
 
   int result = 0;
@@ -756,7 +420,7 @@ static int splt_ogg_process_headers(splt_ogg_state *oggstate, int *error)
   oggstate->cloned_vorbis_comment = SPLT_FALSE;
 
   int packet_err = SPLT_OK;
-  oggstate->headers[0] = splt_ogg_save_packet(&packet, &packet_err);
+  oggstate->headers[0] = splt_ogg_clone_packet(&packet, &packet_err);
   if (packet_err < 0)
   { 
     goto error;
@@ -793,7 +457,7 @@ static int splt_ogg_process_headers(splt_ogg_state *oggstate, int *error)
             goto error_invalid_file;
           }
 
-          oggstate->headers[i+1] = splt_ogg_save_packet(&packet, &packet_err);
+          oggstate->headers[i+1] = splt_ogg_clone_packet(&packet, &packet_err);
           if (packet_err < 0)
           {
             goto error;
@@ -832,6 +496,7 @@ static int splt_ogg_process_headers(splt_ogg_state *oggstate, int *error)
 
 error_invalid_file:
   *error = SPLT_ERROR_INVALID;
+  splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
 error:
   return -1;
 }
@@ -865,6 +530,7 @@ splt_ogg_state *splt_ogg_info(FILE *in, splt_state *state, int *error)
           *error = SPLT_ERROR_WHILE_READING_FILE;
           break;
         default:
+          splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
           *error = SPLT_ERROR_INVALID;
           break;
       }
@@ -876,12 +542,8 @@ splt_ogg_state *splt_ogg_info(FILE *in, splt_state *state, int *error)
   }
 
   /* Read headers in, and save them */
-  if (splt_ogg_process_headers(oggstate, error) == -1)
+  if (splt_ogg_read_headers_and_save_them(state, oggstate, error) == -1)
   {
-    if (*error == SPLT_ERROR_INVALID)
-    {
-      splt_e_set_error_data(state,filename);
-    }
     splt_ogg_v_free(oggstate);
     return NULL;
   }
@@ -897,120 +559,14 @@ splt_ogg_state *splt_ogg_info(FILE *in, splt_state *state, int *error)
   oggstate->cutpoint_begin = 0;
   //what to do if no memory ?
   //- we must free memory from 'vorbis_synthesis_init' ?
-  vorbis_synthesis_init(oggstate->vd,oggstate->vi);
-  vorbis_block_init(oggstate->vd,oggstate->vb);
+  vorbis_synthesis_init(oggstate->vd, oggstate->vi);
+  vorbis_block_init(oggstate->vd, oggstate->vb);
 
   srand(time(NULL));
 
   return oggstate;
 }
 
-static void free_vorbis_comment(vorbis_comment *vc, short cloned_vorbis_comment)
-{
-  if (!vc || cloned_vorbis_comment == 2)
-  {
-    return;
-  }
-
-  vorbis_comment *comment = vc;
-
-  if (!cloned_vorbis_comment)
-  {
-    vorbis_comment_clear(comment);
-    return;
-  }
-
-  long i = 0;
-  for (i = 0;i < comment->comments; i++)
-  {
-    if (comment->user_comments[i])
-    {
-      free(comment->user_comments[i]);
-      comment->user_comments[i] = NULL;
-    }
-  }
-
-  if (comment->user_comments)
-  {
-    free(comment->user_comments);
-    comment->user_comments = NULL;
-  }
-
-  if (comment->comment_lengths)
-  {
-    free(comment->comment_lengths);
-    comment->comment_lengths = NULL;
-  }
-
-  if (comment->vendor)
-  {
-    free(comment->vendor);
-    comment->vendor = NULL;
-  }
-}
-
-static vorbis_comment *clone_vorbis_comment(vorbis_comment *comment)
-{
-  vorbis_comment *cloned_comment = malloc(sizeof(vorbis_comment));
-  if (cloned_comment == NULL)
-  {
-    return NULL;
-  }
-  memset(cloned_comment, 0x0, sizeof(vorbis_comment));
-
-  vorbis_comment_init(cloned_comment);
-
-  int err = splt_su_copy(comment->vendor, &cloned_comment->vendor);
-  if (err < 0)
-  {
-    free(cloned_comment);
-    return NULL;
-  }
-
-  long number_of_comments = comment->comments; 
-  cloned_comment->comments = number_of_comments;
-
-  if (number_of_comments <= 0)
-  {
-    cloned_comment->comment_lengths = NULL;
-    cloned_comment->user_comments = NULL;
-    return cloned_comment;
-  }
-
-  cloned_comment->comment_lengths = malloc(sizeof(int) * number_of_comments);
-  if (cloned_comment->comment_lengths == NULL)
-  {
-    free_vorbis_comment(cloned_comment, SPLT_TRUE);
-    free(cloned_comment);
-    return NULL;
-  }
-  memset(cloned_comment->comment_lengths, 0x0, sizeof(int) * number_of_comments);
-
-  cloned_comment->user_comments = malloc(sizeof(char *) * number_of_comments);
-  if (cloned_comment->user_comments == NULL)
-  {
-    free_vorbis_comment(cloned_comment, SPLT_TRUE);
-    free(cloned_comment);
-    return NULL;
-  }
-  memset(cloned_comment->user_comments, 0x0, sizeof(char *) * number_of_comments);
-
-  int i = 0;
-  for (i = 0;i < number_of_comments; i++)
-  {
-    int err = splt_su_copy(comment->user_comments[i], &cloned_comment->user_comments[i]);
-    if (err < 0)
-    {
-      free_vorbis_comment(cloned_comment, SPLT_TRUE);
-      free(cloned_comment);
-      return NULL;
-    }
-    cloned_comment->comment_lengths[i] = comment->comment_lengths[i];
-  }
-
-  return cloned_comment;
-}
-
 /****************************/
 /* ogg split */
 
@@ -1029,6 +585,14 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
     FILE *in, ogg_int64_t cutpoint, int *error, const char *filename,
     int save_end_point)
 {
+  splt_ogg_new_stream_handler *ogg_new_stream_handler = 
+    splt_ogg_nsh_new(state, oggstate, NULL, NULL, SPLT_FALSE, NULL);
+  if (ogg_new_stream_handler == NULL)
+  {
+    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+    return -1;
+  }
+
   int eos=0;
   ogg_page page;
   ogg_packet packet;
@@ -1056,6 +620,12 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
         //result==1 means that we have a good page
         if (result > 0)
         {
+          if (ogg_page_bos(&page))
+          {
+            splt_ogg_initialise_for_new_stream(ogg_new_stream_handler, 
+                &page, &cutpoint, prevgranpos);
+          }
+
           granpos = ogg_page_granulepos(&page);
 
           /*long page_number = ogg_page_pageno(&page);
@@ -1065,8 +635,9 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
 
           if (ogg_stream_pagein(oggstate->stream_in, &page) == -1)
           {
+            splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
             *error = SPLT_ERROR_INVALID;
-            return -1;
+            goto error;
           }
 
           //for a broken ogg file with no
@@ -1077,10 +648,6 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
             {
               result = ogg_stream_packetout(oggstate->stream_in, &packet);
 
-              /*fprintf(stdout,"packet number = %ld packet granulepos = %ld\n",
-                  packet.packetno, packet.granulepos);
-              fflush(stdout);*/
-
               if (result == 0)
               {
                 break;
@@ -1094,6 +661,17 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
 
               if (result != -1)
               {
+                /*fprintf(stdout,"packet number = %ld packet granulepos = %ld\n",
+                    packet.packetno, packet.granulepos);
+                fflush(stdout);*/
+
+                if (splt_ogg_new_stream_needs_header_packet(ogg_new_stream_handler))
+                {
+                  splt_ogg_new_stream_handle_header_packet(ogg_new_stream_handler, &packet, error);
+                  if (*error < 0) { goto error; }
+                  continue;
+                }
+
                 int bs = splt_ogg_get_blocksize(oggstate, oggstate->vi, &packet);
 
                 cutpoint += splt_ogg_compute_first_granulepos(state, oggstate, &packet, bs);
@@ -1104,8 +682,8 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
                  * just in case.
                  */
                 splt_ogg_free_packet(&oggstate->packets[0]);
-                oggstate->packets[0] = splt_ogg_save_packet(&packet, &packet_err);
-                if (packet_err < 0) { return -1; }
+                oggstate->packets[0] = splt_ogg_clone_packet(&packet, &packet_err);
+                if (packet_err < 0) { goto error; }
               }
             }
 
@@ -1126,6 +704,10 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
               //if == -1, we are out of sync; not a fatal error
               if (result != -1)
               {
+                /*fprintf(stdout,"packet number bis = %ld packet granulepos = %ld\n",
+                    packet.packetno, packet.granulepos);
+                fflush(stdout);*/
+
                 int bs = splt_ogg_get_blocksize(oggstate, oggstate->vi, &packet);
                 prevgranpos += bs;
 
@@ -1144,22 +726,22 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
                 if (prevgranpos > cutpoint)
                 {
                   splt_ogg_free_packet(&oggstate->packets[1]);
-                  oggstate->packets[1] = splt_ogg_save_packet(&packet, &packet_err);
-                  if (packet_err < 0) { return -1; }
+                  oggstate->packets[1] = splt_ogg_clone_packet(&packet, &packet_err);
+                  if (packet_err < 0) { goto error; }
                   eos = 1;
                   break;
                 }
 
                 splt_ogg_free_packet(&oggstate->packets[0]);
-                oggstate->packets[0] = splt_ogg_save_packet(&packet, &packet_err);
-                if (packet_err < 0) { return -1; }
+                oggstate->packets[0] = splt_ogg_clone_packet(&packet, &packet_err);
+                if (packet_err < 0) { goto error; }
               }
             }
           }
 
           if (ogg_page_eos(&page))
           {
-            eos = 1;
+            break;
           }
         }
       }
@@ -1174,7 +756,7 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
       }
       else if (sync_bytes == -1)
       {
-        return -1;
+        goto error;
       }
     }
   }
@@ -1182,14 +764,21 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
   if (granpos < cutpoint)
   {
     *error = SPLT_ERROR_BEGIN_OUT_OF_FILE;
-    return -1;
+    goto error;
   }
 
   /* Remaining samples in first packet */
   oggstate->initialgranpos = prevgranpos - cutpoint;
   oggstate->cutpoint_begin = cutpoint;
 
+  splt_ogg_nsh_free(&ogg_new_stream_handler);
+
   return 0;
+
+error:
+  splt_ogg_nsh_free(&ogg_new_stream_handler);
+
+  return -1;
 }
 
 /* Process second stream.
@@ -1207,12 +796,20 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
     int *error, const char *output_fname, int save_end_point,
     double *sec_split_time_length)
 {
-  splt_c_put_progress_text(state,SPLT_PROGRESS_CREATE);
+  splt_c_put_progress_text(state, SPLT_PROGRESS_CREATE);
 
   //TODO: eos not set on ripped streams on last split file
 
   splt_ogg_state *oggstate = state->codec;
 
+  splt_ogg_new_stream_handler *ogg_new_stream_handler = 
+    splt_ogg_nsh_new(state, oggstate, stream, output_fname, SPLT_TRUE, NULL);
+  if (ogg_new_stream_handler == NULL)
+  {
+    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+    return -1;
+  }
+
   ogg_packet packet;
   ogg_page page;
   int eos=0;
@@ -1318,23 +915,26 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
       {
         if (result != -1)
         {
+          if (ogg_page_bos(&page))
+          {
+            splt_ogg_initialise_for_new_stream(ogg_new_stream_handler, &page, &cutpoint, prev_granpos);
+            oggstate->cutpoint_begin = 0;
+          }
+
           page_granpos = ogg_page_granulepos(&page) - oggstate->cutpoint_begin;
 
-          if (ogg_page_eos(&page)) 
-          {
-            eos = 1;
-          }
+          /*long page_number = ogg_page_pageno(&page);
+          fprintf(stdout,"end point page number = %ld, granule position = %ld, cutpoint = %ld, cutpoint_begin = %ld\n",
+              page_number, page_granpos, cutpoint, oggstate->cutpoint_begin);
+          fflush(stdout);*/
 
           if (ogg_stream_pagein(oggstate->stream_in, &page) == -1)
           {
+            splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
             *error = SPLT_ERROR_INVALID;
-            return -1;
+            goto error;
           }
 
-//          fprintf(stdout, "page_granpos = %ld\n", page_granpos - first_cut_granpos);
-//          fprintf(stdout, "cutpoint = %ld\n", cutpoint - first_cut_granpos);
-//          fflush(stdout);
-
           if ((cutpoint == 0) || (page_granpos < cutpoint))
           {
             while(1)
@@ -1350,6 +950,17 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
               {
                 if (result != -1)
                 {
+                  /*fprintf(stdout,"end point packet number = %ld packet granulepos = %ld, b_o_s = %ld, e_o_s = %ld\n",
+                      packet.packetno, packet.granulepos, packet.b_o_s, packet.e_o_s);
+                  fflush(stdout);*/
+
+                  if (splt_ogg_new_stream_needs_header_packet(ogg_new_stream_handler))
+                  {
+                    splt_ogg_new_stream_handle_header_packet(ogg_new_stream_handler, &packet, error);
+                    if (*error < 0) { goto error; }
+                    continue;
+                  }
+ 
                   int bs = splt_ogg_get_blocksize(oggstate, oggstate->vi, &packet);
                   ogg_int64_t first_granpos = splt_ogg_compute_first_granulepos(state, oggstate, &packet, bs);
                   if (first_cut_granpos == 0)
@@ -1364,16 +975,14 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
 
                   //we need to save the last packet, so save the current packet each time
                   splt_ogg_free_packet(&oggstate->packets[0]);
-                  oggstate->packets[0] = splt_ogg_save_packet(&packet, &packet_err);
+                  oggstate->packets[0] = splt_ogg_clone_packet(&packet, &packet_err);
 
-                  if (packet_err < 0) { return -1; }
+                  if (packet_err < 0) { goto error; }
                   if (current_granpos > page_granpos)
                   {
                     current_granpos = page_granpos;
                   }
                   packet.granulepos = current_granpos;
-//                  fprintf(stdout,"granpos 1 = %ld\n", packet.granulepos);
-//                  fflush(stdout);
                   packet.packetno = packetnum++;
 
                   //progress
@@ -1397,12 +1006,12 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
                   if (packet.packetno == 4 && packet.granulepos != -1)
                   {
                     if (splt_ogg_write_pages_to_file(state, stream,f, 1,
-                          error, output_fname)) { return -1; }
+                          error, output_fname)) { goto error; }
                   }
                   else
                   {
                     if (splt_ogg_write_pages_to_file(state, stream,f, 0,
-                          error, output_fname)) { return -1; }
+                          error, output_fname)) { goto error; }
                   }
                 }
               }
@@ -1439,7 +1048,7 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
               adjust = 0;
               splt_c_put_progress_text(state, SPLT_PROGRESS_CREATE);
 
-              if (*error < 0) { return -1; }
+              if (*error < 0) { goto error; }
             }
             else 
             {
@@ -1451,6 +1060,17 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
               //if == -1, we are out of sync; not a fatal error
               if (result != -1)
               {
+                /*fprintf(stdout,"end point packet bis number = %ld packet granulepos = %ld\n",
+                    packet.packetno, packet.granulepos);
+                fflush(stdout);*/
+
+                if (splt_ogg_new_stream_needs_header_packet(ogg_new_stream_handler))
+                {
+                  splt_ogg_new_stream_handle_header_packet(ogg_new_stream_handler, &packet, error);
+                  if (*error < 0) { goto error; }
+                  continue;
+                }
+
                 int bs;
                 bs = splt_ogg_get_blocksize(oggstate, oggstate->vi, &packet);
 
@@ -1488,9 +1108,9 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
                   //don't save the last packet if exact split
                   if (prev_granpos != cutpoint)
                   {
-                    oggstate->packets[1] = splt_ogg_save_packet(&packet, &packet_err);
+                    oggstate->packets[1] = splt_ogg_clone_packet(&packet, &packet_err);
                   }
-                  if (packet_err < 0) { return -1; }
+                  if (packet_err < 0) { goto error; }
                   if (first_cut_granpos != 0)
                   {
                     packet.granulepos = current_granpos;
@@ -1521,21 +1141,21 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
                 }
 
                 splt_ogg_free_packet(&oggstate->packets[0]);
-                oggstate->packets[0] = splt_ogg_save_packet(&packet, &packet_err);
-                if (packet_err < 0) { return -1; }
+                oggstate->packets[0] = splt_ogg_clone_packet(&packet, &packet_err);
+                if (packet_err < 0) { goto error; }
 
                 ogg_stream_packetin(stream, &packet);
                 if (splt_ogg_write_pages_to_file(state, stream, f, 0, error, output_fname))
                 {
-                  return -1;
+                  goto error;
                 }
               }
             }
           }
 
-          if(ogg_page_eos(&page))
+          if (ogg_page_eos(&page))
           {
-            eos=1;
+            break;
           }
         }
       }
@@ -1550,7 +1170,7 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
       }
       else if (sync_bytes == -1)
       {
-        return -1;
+        goto error;
       }
     }
   }
@@ -1559,17 +1179,19 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
   {
     if (splt_ogg_write_pages_to_file(state, stream, f, 1, error, output_fname))
     {
-      return -1;
+      goto error;
     }
 
     oggstate->end = -1; // No more data available. Next processes aborted
 
+    splt_ogg_nsh_free(&ogg_new_stream_handler);
+
     return 0;
   }
 
   if (splt_ogg_write_pages_to_file(state, stream, f, 0, error, output_fname))
   {
-    return -1;
+    goto error;
   }
 
   oggstate->initialgranpos = prev_granpos - cutpoint;
@@ -1591,11 +1213,17 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
     rewind(oggstate->in);
   }
 
+  splt_ogg_nsh_free(&ogg_new_stream_handler);
+
   return 0;
 
 write_error:
   splt_e_set_error_data(state, output_fname);
   *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
+
+error:
+  splt_ogg_nsh_free(&ogg_new_stream_handler);
+
   return -1;
 }
 
@@ -1607,7 +1235,6 @@ double splt_ogg_split(const char *output_fname, splt_state *state,
   splt_ogg_state *oggstate = state->codec;
 
   ogg_stream_state stream_out;
-  ogg_packet header_comm;
   ogg_int64_t begin, end = 0, cutpoint = 0;
 
   begin = (ogg_int64_t) (sec_begin * oggstate->vi->rate);
@@ -1677,36 +1304,20 @@ double splt_ogg_split(const char *output_fname, splt_state *state,
     }
   }
 
-  /* gets random serial number*/
   ogg_stream_init(&stream_out, rand());
 
-  //vorbis memory leak ?
-  vorbis_commentheader_out(&oggstate->vc, &header_comm);
-
-  int packet_err = SPLT_OK;
-  splt_ogg_free_packet(&oggstate->headers[1]);
-  oggstate->headers[1] = splt_ogg_save_packet(&header_comm, &packet_err);
-  ogg_packet_clear(&header_comm);
-
-  free_vorbis_comment(&oggstate->vc, oggstate->cloned_vorbis_comment);
-  oggstate->cloned_vorbis_comment = 2;
-
-  if (packet_err < 0)
+  splt_ogg_set_tags_in_headers(oggstate, error);
+  if (*error < 0)
   {
-    *error = packet_err;
     ogg_stream_clear(&stream_out);
     return sec_end_time;
   }
 
-  splt_ogg_submit_headers_to_stream(&stream_out, oggstate);
-
-  if (splt_ogg_write_pages_to_file(state, &stream_out, oggstate->out, 1,
-        error, output_fname) == -1)
+  if (splt_ogg_write_header_packets(state, oggstate, &stream_out, output_fname, error) == -1)
   {
     goto end;
   }
 
-  //find end cutpoint and get error
   double sec_split_time_length = sec_end - sec_begin;
   splt_ogg_find_end_cutpoint(state, &stream_out, oggstate->in, 
       oggstate->out, cutpoint, adjust, threshold, error, output_fname,
@@ -1924,7 +1535,13 @@ void splt_pl_set_original_tags(splt_state *state, int *error)
 void splt_pl_clear_original_tags(splt_original_tags *original_tags)
 {
   vorbis_comment *comment = (vorbis_comment *)original_tags->all_original_tags;
-  free_vorbis_comment(comment, SPLT_TRUE);
+  if (!comment)
+  {
+    return;
+  }
+
+  splt_ogg_free_vorbis_comment(comment, SPLT_TRUE);
+
   free(original_tags->all_original_tags);
   original_tags->all_original_tags = NULL;
 }
