@@ -676,6 +676,11 @@ void splt_cue_export_to_file(splt_state *state, const char *out_file,
     splt_cue_write_title_performer(state, file_output, -1, SPLT_TRUE, SPLT_FALSE);
 
     long mins = 0, secs = 0, hundr = 0;
+    if (splitpoint == LONG_MAX)
+    {
+      splitpoint = total_time;
+    }
+
     splt_sp_get_mins_secs_hundr_from_splitpoint(splitpoint, &mins, &secs, &hundr);
     fprintf(file_output, "    INDEX 01 %02ld:%02ld:%02ld\n", mins, secs, hundr);
 
