@@ -3,7 +3,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  *********************************************************/
 
@@ -25,14 +25,21 @@
  * USA.
  *********************************************************/
 
+/*! \file
+
+This file contains all I/O functions this library uses.
+*/
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
 #include <errno.h>
 
+#ifdef __WIN32__
+#include <shlwapi.h>
+#endif
+
 #include "splt.h"
-#include "win32.h"
 
 static int splt_u_fname_is_directory_parent(char *fname, int fname_size);
 
@@ -229,18 +236,45 @@ static int splt_io_linked_file_type_is(const char *fname, int file_type)
 
 int splt_io_check_if_directory(const char *fname)
 {
-  if (fname != NULL)
+#ifdef __WIN32__
+  int is_file = splt_io_check_if_file(NULL, fname);
+  if (is_file)
   {
-#ifndef __WIN32__
-    int is_link = splt_io_file_type_is(fname, S_IFLNK);
+    return SPLT_FALSE;
+  }
+
+  if (splt_w32_check_if_encoding_is_utf8(fname))
+  {
+    wchar_t *wpath = splt_w32_utf8_to_utf16(fname);
+    if (PathFileExistsW(wpath) == 1)
+    {
+      free(wpath);
+      return SPLT_TRUE;
+    }
+
+    free(wpath);
+    return SPLT_FALSE;
+  }
+  else
+  {
+    if (PathFileExistsA(fname) == 1)
+    {
+      return SPLT_TRUE;
+    }
+
+    return SPLT_FALSE;
+  }
 #endif
 
+  if (fname != NULL)
+  {
     if (splt_io_file_type_is(fname, S_IFDIR))
     {
       return SPLT_TRUE;
     }
 
 #ifndef __WIN32__
+    int is_link = splt_io_file_type_is(fname, S_IFLNK);
     if (is_link && splt_io_linked_file_type_is(fname, S_IFDIR))
     {
       return SPLT_TRUE;
@@ -263,16 +297,13 @@ int splt_io_check_if_file(splt_state *state, const char *fname)
       return SPLT_TRUE;
     }
 
-#ifndef __WIN32__
-    int is_link = splt_io_file_type_is(fname, S_IFLNK);
-#endif
-
     if (splt_io_file_type_is(fname, S_IFREG))
     {
       return SPLT_TRUE;
     }
 
 #ifndef __WIN32__
+    int is_link = splt_io_file_type_is(fname, S_IFLNK);
     if (is_link && splt_io_linked_file_type_is(fname, S_IFREG))
     {
       return SPLT_TRUE;
@@ -280,7 +311,10 @@ int splt_io_check_if_file(splt_state *state, const char *fname)
 #endif
   }
 
-  splt_e_set_strerror_msg_with_data(state, fname);
+  if (state != NULL)
+  {
+    splt_e_set_strerror_msg_with_data(state, fname);
+  }
 
   return SPLT_FALSE;
 }
@@ -398,18 +432,13 @@ int splt_io_create_directories(splt_state *state, const char *dir)
 
       if (! splt_io_check_if_directory(dir_to_create))
       {
-        //don't create output directories if we pretend to split
-        //TODO: remove if
-        if (! splt_o_get_int_option(state, SPLT_OPT_PRETEND_TO_SPLIT))
-        {
-          splt_d_print_debug(state,"Creating directory _%s_ ...\n", dir_to_create);
+        splt_d_print_debug(state,"Creating directory _%s_ ...\n", dir_to_create);
 
-          if ((splt_io_mkdir(state, dir_to_create)) == -1)
-          {
-            splt_e_set_strerror_msg_with_data(state, dir_to_create);
-            result = SPLT_ERROR_CANNOT_CREATE_DIRECTORY;
-            goto end;
-          }
+        if ((splt_io_mkdir(state, dir_to_create)) == -1)
+        {
+          splt_e_set_strerror_msg_with_data(state, dir_to_create);
+          result = SPLT_ERROR_CANNOT_CREATE_DIRECTORY;
+          goto end;
         }
       }
     }
@@ -681,7 +710,7 @@ char *splt_io_readline(FILE *stream, int *error)
   }
 
   int err = SPLT_OK;
-  int bufsize = 255;
+  int bufsize = 1024;
   char *buffer = malloc(sizeof(char) * bufsize);
   buffer[0] = '\0';
 
