@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -30,21 +30,22 @@
  *
  *********************************************************/
 
+/*! \file
+
+Loading and unloading of plug-ins
+*/
 #include <string.h>
 #include <dirent.h>
 #include <errno.h>
 
-#include "splt.h"
-#include "plugins.h"
+#include <ltdl.h>
 
 #ifdef __WIN32__
 #include <direct.h>
-#include "win32.h"
 #endif
 
-static int splt_p_alloc_init_new_plugin(splt_plugins *pl);
-static void splt_p_free_plugin_data_info(splt_plugin_data *pl_data);
-static void splt_p_free_plugin_data(splt_plugin_data *pl_data);
+#include "splt.h"
+#include "plugins.h"
 
 int splt_p_append_plugin_scan_dir(splt_state *state, const char *dir)
 {
@@ -83,68 +84,155 @@ int splt_p_append_plugin_scan_dir(splt_state *state, const char *dir)
   return SPLT_OK;
 }
 
-//function to filter the plugin files
-static int splt_p_filter_plugin_files(const struct dirent *de)
-{
-  char *file = (char *) de->d_name;
-  const char *p_end = NULL;
-  const char *p_start = NULL;
-  if (file)
-  {
-    if (strlen(file) >= 8)
-    {
-      //if the name starts with splt_and contains .so or .sl or .dll or .dylib
-      if (strncmp(file,"libsplt_",8) == 0)
-      {
-        splt_d_print_debug(NULL, "Looking at the file _%s_\n", file);
-
-        p_start = strchr(file,'.');
-
-#ifndef __WIN32__
-        //gnu/linux .so.0
-        p_end = strstr(file, ".so.0");
-        if (p_end != NULL && (p_start == p_end) && (*(p_end+5) == '\0'))
-        {
-          return 1;
-        }
-#endif
-
-        p_end = strrchr(file,'.');
-        if ((p_end != NULL) && (p_start == p_end))
-        {
-          //windows .dll
 #ifdef __WIN32__
-          if (strcmp(p_end,".dll") == 0)
-          {
-            return 1;
-          }
-#else
-          //bsd .sl & darwin .dylib
-          if ((strcmp(p_end,".sl") == 0) || (strcmp(p_end,".dylib") == 0))
-          {
-            return 1;
-          }
-#endif
-        }
-      }
+static int splt_p_filter_plugin_files(const struct _wdirent *de)
+{
+  wchar_t *file = (wchar_t *) de->d_name;
+  const wchar_t *p_end = NULL;
+  const wchar_t *p_start = NULL;
+
+  if (!file)
+  {
+    return 0;
+  }
+
+  if (wcslen(file) < 8)
+  {
+    return 0;
+  }
+
+  if (wcsncmp(file, L"libsplt_", 8) != 0)
+  {
+    return 0;
+  }
+
+  splt_d_print_debug(NULL, "Looking at the file _%s_\n", splt_w32_utf16_to_utf8(file));
+
+  p_start = wcschr(file,'.');
+  p_end = wcsrchr(file,'.');
+  if ((p_end != NULL) && (p_start == p_end))
+  {
+    if (wcscmp(p_end, L".dll") == 0)
+    {
+      return 1;
     }
   }
 
   return 0;
 }
+#else
+static int splt_p_filter_plugin_files(const struct dirent *de)
+{
+  char *file = (char *) de->d_name;
+  const char *p_end = NULL;
+  const char *p_start = NULL;
 
-//scans the directory *directory for plugins
-//-directory must not be NULL
+  if (!file)
+  {
+    return 0;
+  }
+
+  if (strlen(file) < 8)
+  {
+    return 0;
+  }
+
+  if (strncmp(file,"libsplt_", 8) != 0)
+  {
+    return 0;
+  }
+
+  splt_d_print_debug(NULL, "Looking at the file _%s_\n", file);
+
+  p_start = strchr(file,'.');
+
+#ifndef __WIN32__
+  //gnu/linux .so.0
+  p_end = strstr(file, ".so.0");
+  if (p_end != NULL && (p_start == p_end) && (*(p_end+5) == '\0'))
+  {
+    return 1;
+  }
+#endif
+
+  p_end = strrchr(file,'.');
+  if ((p_end != NULL) && (p_start == p_end))
+  {
+    //windows .dll
+#ifdef __WIN32__
+    if (strcmp(p_end,".dll") == 0)
+    {
+      return 1;
+    }
+#else
+    //bsd .sl & darwin .dylib
+    if ((strcmp(p_end,".sl") == 0) || (strcmp(p_end,".dylib") == 0))
+    {
+      return 1;
+    }
+#endif
+  }
+
+  return 0;
+}
+#endif
+
+static int splt_p_alloc_init_new_plugin(splt_plugins *pl)
+{
+  int return_value = SPLT_OK;
+
+  if (pl->data == NULL)
+  {
+    pl->data = malloc(sizeof(splt_plugin_data) * (pl->number_of_plugins_found+1));
+    if (pl->data == NULL)
+    {
+      return_value = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+      return return_value;
+    }
+  }
+  else
+  {
+    pl->data = realloc(pl->data,sizeof(splt_plugin_data) *
+        (pl->number_of_plugins_found+1));
+    if (pl->data == NULL)
+    {
+      return_value = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+      return return_value;
+    }
+  }
+
+  pl->data[pl->number_of_plugins_found].func = NULL;
+  pl->data[pl->number_of_plugins_found].plugin_handle = NULL;
+  pl->data[pl->number_of_plugins_found].info.version = 0;
+  pl->data[pl->number_of_plugins_found].info.name = NULL;
+  pl->data[pl->number_of_plugins_found].info.extension = NULL;
+  pl->data[pl->number_of_plugins_found].info.upper_extension = NULL;
+  pl->data[pl->number_of_plugins_found].plugin_filename = NULL;
+
+  return return_value;
+}
+
+/*! scans the directory *directory for plugins
+
+directory must not be NULL
+*/
 static int splt_p_scan_dir_for_plugins(splt_state *state, splt_plugins *pl, const char *directory)
 {
   int return_value = SPLT_OK;
 
+#ifdef __WIN32__
+  struct _wdirent **files = NULL;
+#else
   struct dirent **files = NULL;
+#endif
 
   int number_of_files = 0;
   errno = 0;
+#ifdef __WIN32__
+  number_of_files = wscandir(directory, &files, splt_p_filter_plugin_files, walphasort);
+#else
   number_of_files = scandir(directory, &files, splt_p_filter_plugin_files, alphasort);
-  int directory_len = strlen(directory);
+#endif
   int new_number_of_files = number_of_files;
 
   if (number_of_files == -1)
@@ -156,35 +244,32 @@ static int splt_p_scan_dir_for_plugins(splt_state *state, splt_plugins *pl, cons
   }
   else if (new_number_of_files >= 0)
   {
-    char *fname = NULL;
+    char *dir_and_fname = NULL;
 
     //for all the filtered found plugins,
     //copy their name
     while (new_number_of_files--)
     {
-      fname = files[new_number_of_files]->d_name;
-      int fname_len = strlen(fname);
+#ifdef __WIN32__
+      char *fname = splt_w32_utf16_to_utf8(files[new_number_of_files]->d_name);
+#else
+      char *fname = files[new_number_of_files]->d_name;
+#endif
 
-      //get the full directory + filename
-      int dir_and_fname_len = fname_len + directory_len + 3;
-      char *dir_and_fname = malloc(sizeof(char) * dir_and_fname_len);
-      if (dir_and_fname == NULL)
-      {
-        return_value = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-        goto end;
-      }
-      snprintf(dir_and_fname, dir_and_fname_len - 1, "%s%c%s",directory,SPLT_DIRCHAR,fname);
+#ifdef __WIN32__
+      splt_su_copy(fname, &dir_and_fname);
+#else
+      splt_su_copy(directory, &dir_and_fname);
+      splt_su_append_str(&dir_and_fname, SPLT_DIRSTR, fname, NULL);
+#endif
 
-      //check if we already have a plugin with the same file
-      int we_already_have_this_plugin_file = SPLT_FALSE;
       int i = 0;
       int err = SPLT_OK;
       for (i = 0;i < pl->number_of_plugins_found;i++)
       {
         if (splt_check_is_the_same_file(state, dir_and_fname, pl->data[i].plugin_filename, &err))
         {
-          we_already_have_this_plugin_file = SPLT_TRUE;
-          break;
+          goto loop_end;
         }
 
         if (err != SPLT_OK)
@@ -194,53 +279,41 @@ static int splt_p_scan_dir_for_plugins(splt_state *state, splt_plugins *pl, cons
         }
       }
 
-      //if we don't have a plugin from the same file, add this new plugin
-      if (! we_already_have_this_plugin_file)
+      int alloc_err = splt_p_alloc_init_new_plugin(pl);
+      if (alloc_err < 0)
       {
-        int alloc_err = splt_p_alloc_init_new_plugin(pl);
-        if (alloc_err < 0)
-        {
-          return_value = alloc_err;
-          goto end;
-        }
-
-        pl->data[pl->number_of_plugins_found].func = malloc(sizeof(splt_plugin_func));
-        if (pl->data[pl->number_of_plugins_found].func == NULL)
-        {
-          return_value = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-          goto end;
-        }
-        memset(pl->data[pl->number_of_plugins_found].func,0,sizeof(splt_plugin_func));
-
-        pl->data[pl->number_of_plugins_found].plugin_filename = malloc(sizeof(char) *
-            dir_and_fname_len);
-        if (pl->data[pl->number_of_plugins_found].plugin_filename == NULL)
-        {
-          return_value = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-          goto end;
-        }
-
-        //set the plugin path
-        snprintf(pl->data[pl->number_of_plugins_found].plugin_filename,
-            dir_and_fname_len - 1,"%s%c%s",directory,SPLT_DIRCHAR,fname);
-
-        pl->number_of_plugins_found++;
+        return_value = alloc_err;
+        goto end;
       }
 
-      //free some memory
-      if (dir_and_fname)
+      pl->data[pl->number_of_plugins_found].func = malloc(sizeof(splt_plugin_func));
+      if (pl->data[pl->number_of_plugins_found].func == NULL)
       {
-        free(dir_and_fname);
-        dir_and_fname = NULL;
+        return_value = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+        goto end;
       }
+      memset(pl->data[pl->number_of_plugins_found].func, 0, sizeof(splt_plugin_func));
+
+      splt_su_copy(dir_and_fname, 
+          &pl->data[pl->number_of_plugins_found].plugin_filename);
+
+      pl->number_of_plugins_found++;
+
+loop_end:
+      ;
     }
 
 end:
     ;
 
+    if (dir_and_fname)
+    {
+      free(dir_and_fname);
+      dir_and_fname = NULL;
+    }
+
     if (files)
     {
-      //free memory
       while (number_of_files--)
       {
         if (files[number_of_files])
@@ -298,6 +371,46 @@ static int splt_p_find_plugins(splt_state *state)
   return return_value;
 }
 
+//! Used by splt_p_free_plugin_data
+static void splt_p_free_plugin_data_info(splt_plugin_data *pl_data)
+{
+  if (pl_data->info.name)
+  {
+    free(pl_data->info.name);
+    pl_data->info.name = NULL;
+  }
+  if (pl_data->info.extension)
+  {
+    free(pl_data->info.extension);
+    pl_data->info.extension = NULL;
+  }
+  if (pl_data->info.upper_extension)
+  {
+    free(pl_data->info.upper_extension);
+    pl_data->info.upper_extension = NULL;
+  }
+}
+
+static void splt_p_free_plugin_data(splt_plugin_data *pl_data)
+{
+  splt_p_free_plugin_data_info(pl_data);
+  if (pl_data->plugin_filename)
+  {
+    free(pl_data->plugin_filename);
+    pl_data->plugin_filename = NULL;
+  }
+  if (pl_data->plugin_handle)
+  {
+    lt_dlclose(pl_data->plugin_handle);
+    pl_data->plugin_handle = NULL;
+  }
+  if (pl_data->func)
+  {
+    free(pl_data->func);
+    pl_data->func = NULL;
+  }
+}
+
 int splt_p_move_replace_plugin_data(splt_state *state, int old, int new)
 {
   splt_plugins *pl = state->plug;
@@ -444,27 +557,16 @@ static int splt_p_open_get_valid_plugins(splt_state *state)
   {
     int index_to_remove = plugin_index_to_remove[i] - left_shift;
 
-    splt_d_print_debug(state,"Removing the plugin _%s_\n",
-        pl->data[index_to_remove].plugin_filename);
+    splt_d_print_debug(state,"Removing the plugin _%s_ at index %d\n",
+        pl->data[index_to_remove].plugin_filename, index_to_remove);
 
-    if (i == number_of_plugins_to_remove-1)
-    {
-      splt_p_free_plugin_data(&pl->data[index_to_remove]);
-    }
-    else
-    {
-      error = splt_p_shift_left_plugins_data(state, index_to_remove);
-      if (error != SPLT_OK)
-      {
-        goto end;
-      }
-    }
+    error = splt_p_shift_left_plugins_data(state, index_to_remove);
+    if (error < SPLT_OK) { break; }
 
     pl->number_of_plugins_found--;
     left_shift++;
   }
 
-end:
   //free the plugin index to remove
   if (plugin_index_to_remove)
   {
@@ -501,7 +603,7 @@ int splt_p_find_get_plugins_data(splt_state *state)
   if (return_value >= 0)
   {
     splt_plugins *pl = state->plug;
-    splt_d_print_debug(state,"\nNumber of plugins found: _%d_\n",pl->number_of_plugins_found);
+    splt_d_print_debug(state,"\nNumber of plugins found: _%d_\n", pl->number_of_plugins_found);
     int i = 0;
     int err = 0;
     for (i = 0;i < pl->number_of_plugins_found;i++)
@@ -524,8 +626,12 @@ int splt_p_find_get_plugins_data(splt_state *state)
         lt_dlsym(pl->data[i].plugin_handle, "splt_pl_end");
       pl->data[i].func->scan_silence =
         lt_dlsym(pl->data[i].plugin_handle, "splt_pl_scan_silence");
+      pl->data[i].func->scan_trim_silence =
+        lt_dlsym(pl->data[i].plugin_handle, "splt_pl_scan_trim_silence");
       pl->data[i].func->set_original_tags =
         lt_dlsym(pl->data[i].plugin_handle, "splt_pl_set_original_tags");
+      pl->data[i].func->clear_original_tags =
+        lt_dlsym(pl->data[i].plugin_handle, "splt_pl_clear_original_tags");
       pl->data[i].func->set_plugin_info =
         lt_dlsym(pl->data[i].plugin_handle, "splt_pl_set_plugin_info");
       if (pl->data[i].func->set_plugin_info != NULL)
@@ -539,10 +645,13 @@ int splt_p_find_get_plugins_data(splt_state *state)
       {
         splt_d_print_debug(state,"plugin filename = _%s_\n", pl->data[i].plugin_filename);
       }
+
       const char *temp = splt_p_get_name(state, &err);
       splt_d_print_debug(state,"plugin name = _%s_\n", temp);
+
       float version = splt_p_get_version(state, &err);
-      splt_d_print_debug(state,"plugin version = _%d_\n", version);
+      splt_d_print_debug(state,"plugin version = _%lf_\n", version);
+
       temp = splt_p_get_extension(state,&err);
       splt_d_print_debug(state,"extension = _%s_\n\n", temp);
     }
@@ -815,6 +924,30 @@ int splt_p_scan_silence(splt_state *state, int *error)
   return 0;
 }
 
+int splt_p_scan_trim_silence(splt_state *state, int *error)
+{
+  splt_plugins *pl = state->plug;
+  int current_plugin = splt_p_get_current_plugin(state);
+  if ((current_plugin < 0) || (current_plugin >= pl->number_of_plugins_found))
+  {
+    *error = SPLT_ERROR_NO_PLUGIN_FOUND;
+    return 0;
+  }
+  else
+  {
+    if (pl->data[current_plugin].func->scan_trim_silence != NULL)
+    {
+      return pl->data[current_plugin].func->scan_trim_silence(state, error);
+    }
+    else
+    {
+      *error = SPLT_PLUGIN_ERROR_UNSUPPORTED_FEATURE;
+    }
+  }
+
+  return 0;
+}
+
 void splt_p_set_original_tags(splt_state *state, int *error)
 {
   splt_plugins *pl = state->plug;
@@ -833,6 +966,24 @@ void splt_p_set_original_tags(splt_state *state, int *error)
   }
 }
 
+void splt_p_clear_original_tags(splt_state *state, int *error)
+{
+  splt_plugins *pl = state->plug;
+  int current_plugin = splt_p_get_current_plugin(state);
+  if ((current_plugin < 0) || (current_plugin >= pl->number_of_plugins_found))
+  {
+    *error = SPLT_ERROR_NO_PLUGIN_FOUND;
+    return;
+  }
+  else
+  {
+    if (pl->data[current_plugin].func->clear_original_tags != NULL)
+    {
+      pl->data[current_plugin].func->clear_original_tags(&state->original_tags);
+    }
+  }
+}
+
 static int splt_p_set_default_plugins_scan_dirs(splt_state *state)
 {
   int err = SPLT_OK;
@@ -873,80 +1024,6 @@ int splt_p_set_default_values(splt_state *state)
   return splt_p_set_default_plugins_scan_dirs(state);
 }
 
-static int splt_p_alloc_init_new_plugin(splt_plugins *pl)
-{
-  int return_value = SPLT_OK;
-
-  if (pl->data == NULL)
-  {
-    pl->data = malloc(sizeof(splt_plugin_data) * (pl->number_of_plugins_found+1));
-    if (pl->data == NULL)
-    {
-      return_value = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-      return return_value;
-    }
-  }
-  else
-  {
-    pl->data = realloc(pl->data,sizeof(splt_plugin_data) *
-        (pl->number_of_plugins_found+1));
-    if (pl->data == NULL)
-    {
-      return_value = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-      return return_value;
-    }
-  }
-
-  pl->data[pl->number_of_plugins_found].func = NULL;
-  pl->data[pl->number_of_plugins_found].plugin_handle = NULL;
-  pl->data[pl->number_of_plugins_found].info.version = 0;
-  pl->data[pl->number_of_plugins_found].info.name = NULL;
-  pl->data[pl->number_of_plugins_found].info.extension = NULL;
-  pl->data[pl->number_of_plugins_found].info.upper_extension = NULL;
-  pl->data[pl->number_of_plugins_found].plugin_filename = NULL;
-
-  return return_value;
-}
-
-static void splt_p_free_plugin_data_info(splt_plugin_data *pl_data)
-{
-  if (pl_data->info.name)
-  {
-    free(pl_data->info.name);
-    pl_data->info.name = NULL;
-  }
-  if (pl_data->info.extension)
-  {
-    free(pl_data->info.extension);
-    pl_data->info.extension = NULL;
-  }
-  if (pl_data->info.upper_extension)
-  {
-    free(pl_data->info.upper_extension);
-    pl_data->info.upper_extension = NULL;
-  }
-}
-
-static void splt_p_free_plugin_data(splt_plugin_data *pl_data)
-{
-  splt_p_free_plugin_data_info(pl_data);
-  if (pl_data->plugin_filename)
-  {
-    free(pl_data->plugin_filename);
-    pl_data->plugin_filename = NULL;
-  }
-  if (pl_data->plugin_handle)
-  {
-    lt_dlclose(pl_data->plugin_handle);
-    pl_data->plugin_handle = NULL;
-  }
-  if (pl_data->func)
-  {
-    free(pl_data->func);
-    pl_data->func = NULL;
-  }
-}
-
 void splt_p_free_plugins(splt_state *state)
 {
   splt_plugins *pl = state->plug;
