@@ -53,7 +53,15 @@ static long splt_s_real_split(double splt_beg, double splt_end,
   {
     double new_sec_end_point = 
       splt_p_split(state, final_fname, splt_beg, splt_end, error, save_end_point);
-    new_end_point = splt_co_time_to_long_ceil(new_sec_end_point);
+
+    if (new_sec_end_point == -1.0)
+    {
+      new_end_point = LONG_MAX;
+    }
+    else
+    {
+      new_end_point = splt_co_time_to_long_ceil(new_sec_end_point);
+    }
 
     if (*error >= 0)
     {
@@ -108,8 +116,7 @@ static long splt_s_split(splt_state *state, int first_splitpoint,
         //LONG_MAX == EOF
         if (split_end == LONG_MAX)
         {
-          //Warning : we might not always have total time with input not seekable
-          splt_end = splt_t_get_total_time_as_double_secs(state);
+          splt_end = -1;
         }
         else
         {
@@ -155,6 +162,7 @@ void splt_s_multiple_split(splt_state *state, int *error)
 
   int i = 0;
   int number_of_splitpoints = splt_t_get_splitnumber(state);
+
   while (i  < number_of_splitpoints - 1)
   {
     splt_t_set_current_split(state, i);
@@ -367,7 +375,7 @@ static void splt_s_split_by_time(splt_state *state, int *error,
     int err = SPLT_OK;
 
     int temp_int = number_of_files + 1;
-    if (number_of_files == -1)
+    if (total_time > 0 && number_of_files == -1)
     {
       temp_int = (int)floor(((total_time / 100.0) / (split_time_length))+1) + 1;
     }
@@ -393,8 +401,6 @@ static void splt_s_split_by_time(splt_state *state, int *error,
         save_end_point = SPLT_FALSE;
       }
 
-      int last_file = SPLT_FALSE;
-
       splt_array *new_end_points = splt_array_new();
 
       do {
@@ -412,16 +418,14 @@ static void splt_s_split_by_time(splt_state *state, int *error,
           splt_sp_set_splitpoint_value(state, current_split,
               splt_co_time_to_long_ceil(begin));
           long end_splitpoint = splt_co_time_to_long_ceil(end);
-          if (total_time > 0 && end_splitpoint >= total_time)
-          {
-            end_splitpoint = total_time;
-            //avoid worst scenarios where floor & SPLT_OK_SPLIT_EOF do not work
-            last_file = SPLT_TRUE;
-          }
           splt_sp_set_splitpoint_value(state, current_split+1, end_splitpoint);
 
-          double overlapped_end = (double)
-            ((double)splt_sp_overlap_time(state, current_split+1) / 100.0);
+          long overlapped_time = splt_sp_overlap_time(state, current_split+1);
+          double overlapped_end = -1;
+          if (overlapped_time != LONG_MAX)
+          {
+            overlapped_end = (double) ((double) overlapped_time / 100.0);
+          }
 
           err = splt_u_finish_tags_and_put_output_format_filename(state, -1);
           if (err < 0) { *error = err; break; }
@@ -431,7 +435,15 @@ static void splt_s_split_by_time(splt_state *state, int *error,
 
           double new_sec_end_point = splt_p_split(state, final_fname,
               begin, overlapped_end, error, save_end_point);
-          long new_end_point = splt_co_time_to_long_ceil(new_sec_end_point);
+          long new_end_point = 0;
+          if (new_sec_end_point == -1.0)
+          {
+            new_end_point = LONG_MAX;
+          }
+          else
+          {
+            new_end_point = splt_co_time_to_long_ceil(new_sec_end_point);
+          }
 
           int end_point_index = current_split + 1;
           splt_pair *index_end_point =
@@ -469,11 +481,6 @@ static void splt_s_split_by_time(splt_state *state, int *error,
             free(final_fname);
             final_fname = NULL;
           }
-
-          if (last_file)
-          {
-            break;
-          }
         }
         else
         {
@@ -750,12 +757,13 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
   {
     splt_c_put_info_message_to_client(state, 
         _(" Silence split type: %s mode (Th: %.1f dB,"
-          " Off: %.2f, Min: %.2f, Remove: %s)\n"),
+          " Off: %.2f, Min: %.2f, Remove: %s, Min track: %.2f)\n"),
         auto_user_str,
         splt_o_get_float_option(state, SPLT_OPT_PARAM_THRESHOLD),
         splt_o_get_float_option(state, SPLT_OPT_PARAM_OFFSET),
         splt_o_get_float_option(state, SPLT_OPT_PARAM_MIN_LENGTH),
-        remove_str);
+        remove_str,
+        splt_o_get_float_option(state, SPLT_OPT_PARAM_MIN_TRACK_LENGTH));
   }
  
   short read_silence_from_logs = SPLT_FALSE;
@@ -828,68 +836,83 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
         }
       }
 
-      //put first splitpoint
-      append_error = splt_sp_append_splitpoint(state, 0, NULL, SPLT_SPLITPOINT);
-      if (append_error != SPLT_OK)
-      {
-        *error = append_error;
-      }
-      else
-      {
-        temp = state->silence_list;
-        int i;
+      temp = state->silence_list;
 
-        //we take all splitpoints found and we remove silence 
-        //if needed
-        for (i = 1; i < found; i++)
+      int i;
+
+      //we take all splitpoints found and we remove silence 
+      //if needed
+      for (i = 1; i < found; i++)
+      {
+        if (temp == NULL)
         {
-          if (temp == NULL)
-          {
-            found = i;
-            break;
-          }
-
-          if (splt_o_get_int_option(state, SPLT_OPT_PARAM_REMOVE_SILENCE))
-          {
-            append_error = splt_sp_append_splitpoint(state, 0, NULL, SPLT_SKIPPOINT);
-            if (append_error < 0) { *error = append_error; found = i; break;}
-            append_error = splt_sp_append_splitpoint(state, 0, NULL, SPLT_SPLITPOINT);
-            if (append_error < 0) { *error = append_error; found = i; break;}
-            splt_sp_set_splitpoint_value(state, 2*i-1, splt_co_time_to_long(temp->begin_position));
-            splt_sp_set_splitpoint_value(state, 2*i, splt_co_time_to_long(temp->end_position));
-          }
-          else
-          {
-            long temp_silence_pos = splt_siu_silence_position(temp, offset) * 100;
-            append_error = splt_sp_append_splitpoint(state, temp_silence_pos, NULL, SPLT_SPLITPOINT);
-            if (append_error != SPLT_OK) { *error = append_error; found = i; break; }
-          }
-          temp = temp->next;
+          found = i;
+          break;
         }
 
-        //we order the splitpoints
+        long end_track_point = 0;
+        long end_track_point_after_silence = 0;
         if (splt_o_get_int_option(state, SPLT_OPT_PARAM_REMOVE_SILENCE))
         {
-          splitpoints_appended = (found-1)*2+1;
+          end_track_point = splt_co_time_to_long(temp->begin_position);
+          end_track_point_after_silence = splt_co_time_to_long(temp->end_position);
         }
         else 
         {
-          splitpoints_appended = found;
+          end_track_point = splt_co_time_to_long(splt_siu_silence_position(temp, offset));
         }
 
-        splt_d_print_debug(state,"Order splitpoints...\n");
-        int err = splt_sp_order_splitpoints(state, splitpoints_appended);
-        if (err < 0) { *error = err; }
-
-        if (*error >= 0)
+        if (i == 1)
         {
-          //last splitpoint, end of file
-          append_error =
-            splt_sp_append_splitpoint(state, splt_t_get_total_time(state),
-                NULL, SPLT_SPLITPOINT);
-          if (append_error != SPLT_OK) { *error = append_error; }
+          append_error = splt_sp_append_splitpoint(state, 0, NULL, SPLT_SPLITPOINT);
+          if (append_error < 0) { *error = append_error; found = i; break;}
         }
+
+        if (splt_o_get_int_option(state, SPLT_OPT_PARAM_REMOVE_SILENCE))
+        {
+          append_error = splt_sp_append_splitpoint(state, end_track_point, NULL, SPLT_SKIPPOINT);
+          if (append_error < 0) { *error = append_error; found = i; break;}
+          append_error =
+            splt_sp_append_splitpoint(state, end_track_point_after_silence, NULL, SPLT_SPLITPOINT);
+          if (append_error < 0) { *error = append_error; found = i; break;}
+        }
+        else
+        {
+          append_error = splt_sp_append_splitpoint(state, end_track_point, NULL, SPLT_SPLITPOINT);
+          if (append_error != SPLT_OK) { *error = append_error; found = i; break; }
+        }
+
+        temp = temp->next;
       }
+
+      //we order the splitpoints
+      if (splt_o_get_int_option(state, SPLT_OPT_PARAM_REMOVE_SILENCE))
+      {
+        splitpoints_appended = (found-1)*2+1;
+      }
+      else 
+      {
+        splitpoints_appended = found;
+      }
+
+      splt_d_print_debug(state,"Order splitpoints...\n");
+      splt_sp_order_splitpoints(state, splitpoints_appended);
+
+      if (*error >= 0)
+      {
+        long total_time = splt_t_get_total_time(state);
+        if (total_time <= 0)
+        {
+          total_time = LONG_MAX;
+        }
+
+        //last splitpoint, end of file
+        append_error =
+          splt_sp_append_splitpoint(state, total_time, NULL, SPLT_SPLITPOINT);
+        if (append_error != SPLT_OK) { *error = append_error; }
+      }
+
+      splt_sp_skip_minimum_track_length_splitpoints(state, error);
     }
     else
     {
@@ -897,7 +920,7 @@ int splt_s_set_silence_splitpoints(splt_state *state, int *error)
     }
 
     //if splitpoints are found
-    if ((found > 0) && !we_read_silence_from_logs)
+    if ((*error >= 0) && (found > 0) && !we_read_silence_from_logs)
     {
       //if we write the silence points log file
       if (splt_o_get_int_option(state, SPLT_OPT_ENABLE_SILENCE_LOG))
