@@ -82,7 +82,7 @@ void splt_tu_auto_increment_tracknumber(splt_state *state)
   int current_split = splt_t_get_current_split_file_number(state) - 1;
   int old_current_split = current_split;
 
-  int remaining_tags_like_x = splt_o_get_int_option(state, SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X); 
+  int remaining_tags_like_x = splt_o_get_int_option(state, SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X);
   if (remaining_tags_like_x != -1)
   {
     if (current_split >= state->split.real_tagsnumber)
@@ -449,11 +449,47 @@ void *splt_tu_get_original_tags_data(splt_state *state)
   return state->original_tags.all_original_tags;
 }
 
+splt_tags *splt_tu_get_original_tags_tags(splt_state *state)
+{
+  return &state->original_tags.tags;
+}
+
 static char *splt_tu_get_replaced_with_tags(const char *word,
-    const splt_tags *tags, int track, int *error, int replace_tags_in_tags)
+    const splt_tags *tags, const splt_tags *original_tags,
+    int track, int *error, int replace_tags_in_tags, 
+    int current_split, splt_state *state)
 {
   int err = SPLT_OK;
 
+  if (current_split == -1)
+  {
+    current_split = splt_t_get_current_split_file_number(state) - 1;
+  }
+
+  long mins = -1; long secs = -1; long hundr = -1;
+  long point_value = splt_sp_get_splitpoint_value(state, current_split, &err);
+  splt_co_get_mins_secs_hundr(point_value, &mins, &secs, &hundr);
+  long next_mins = -1; long next_secs = -1; long next_hundr = -1;
+  long next_point_value = -1;
+  if (splt_sp_splitpoint_exists(state, current_split + 1))
+  {
+    next_point_value = splt_sp_get_splitpoint_value(state, current_split + 1, &err);
+    long total_time = splt_t_get_total_time(state);
+    if (total_time > 0 && next_point_value > total_time)
+    {
+      next_point_value = total_time;
+    }
+    splt_co_get_mins_secs_hundr(next_point_value, &next_mins, &next_secs, &next_hundr);
+  }
+  short write_eof = SPLT_FALSE;
+  if (next_point_value == LONG_MAX)
+  {
+    write_eof = SPLT_TRUE;
+  }
+
+  long mMsShH_value = 1;
+  short eof_written = SPLT_FALSE;
+
   char *word_with_tags = NULL;
 
   if (!replace_tags_in_tags)
@@ -470,18 +506,31 @@ static char *splt_tu_get_replaced_with_tags(const char *word,
     return NULL;
   }
 
-  const char *title = tags->title;
-  const char *artist = tags->artist;
-  const char *album= tags->album;
-  const char *performer = tags->performer;
-  const char *year = tags->year;
-  const char *comment = tags->comment;
+  const splt_tags *tags_to_replace = tags;
 
   int counter = 0;
   const char *ptr = NULL;
   for (ptr = word; *ptr != '\0'; ptr++)
   {
     if (*ptr == '@')
+    {
+      tags_to_replace = tags;
+    }
+    else if (*ptr == '#')
+    {
+      tags_to_replace = original_tags;
+    }
+
+    const char *title = tags_to_replace->title;
+    const char *artist = tags_to_replace->artist;
+    const char *album= tags_to_replace->album;
+    const char *performer = tags_to_replace->performer;
+    const char *year = tags_to_replace->year;
+    const char *comment = tags_to_replace->comment;
+    const char *genre = tags_to_replace->genre;
+    int real_track = tags_to_replace->track;
+
+    if (*ptr == '@' || *ptr == '#')
     {
       err = splt_su_append(&word_with_tags, buffer, counter, NULL);
       if (err != SPLT_OK) { goto error; }
@@ -493,6 +542,57 @@ static char *splt_tu_get_replaced_with_tags(const char *word,
 
       switch (*ptr)
       {
+        case 's':
+          mMsShH_value = secs;
+          goto put_value;
+        case 'S':
+          mMsShH_value = next_secs;
+          goto put_value;
+        case 'm':
+          mMsShH_value = mins;
+          goto put_value;
+        case 'M':
+          mMsShH_value = next_mins;
+          goto put_value;
+        case 'h':
+          mMsShH_value = hundr;
+          goto put_value;
+        case 'H':
+          mMsShH_value = next_hundr;
+put_value:
+          if (!eof_written)
+          {
+            if (write_eof &&
+                (*ptr == 'S' || *ptr == 'M' || *ptr == 'H'))
+            {
+              write_eof = SPLT_FALSE;
+              eof_written = SPLT_TRUE;
+
+              err = splt_su_append_str(&word_with_tags, "EOF", NULL);
+              if (err != SPLT_OK) { goto error; }
+            }
+            else if (mMsShH_value != -1)
+            {
+              char temp[6] = { '\0' };
+              temp[0] = '%';
+              temp[1] = '0';
+              char number_of_digits = '2';
+              if (*ptr == 'M' || *ptr == 'm')
+              {
+                number_of_digits = splt_of_get_number_of_digits_from_total_time(state);
+              }
+              temp[2] = number_of_digits;
+              temp[3] = 'l';
+              temp[4] = 'd';
+
+              char mMsShH_str[100] = { '\0' };
+              snprintf(mMsShH_str, 100, temp, mMsShH_value);
+
+              err = splt_su_append_str(&word_with_tags, mMsShH_str, NULL);
+              if (err != SPLT_OK) { goto error; }
+            }
+          }
+          break;
         case 'a':
           if (artist != NULL)
           {
@@ -528,6 +628,13 @@ static char *splt_tu_get_replaced_with_tags(const char *word,
             if (err != SPLT_OK) { goto error; }
           }
           break;
+        case 'g':
+          if (genre != NULL)
+          {
+            err = splt_su_append_str(&word_with_tags, genre, NULL);
+            if (err != SPLT_OK) { goto error; }
+          }
+          break;
         case 'y':
           if (year != NULL)
           {
@@ -538,8 +645,22 @@ static char *splt_tu_get_replaced_with_tags(const char *word,
         case 'N':
         case 'n':
           ;
+          int track_to_set = track;
+
+          if (*ptr == 'n')
+          {
+            track_to_set = real_track;
+          }
+
+          char temp[5] = { '\0' };
+          temp[0] = '%';
+          temp[1] = '0';
+          temp[2] = splt_of_get_oformat_number_of_digits_as_char(state);
+          temp[3] = 'd';
+
           char track_str[10] = { '\0' };
-          snprintf(track_str, 10, "%d",track);
+          snprintf(track_str, 10, temp, track_to_set);
+
           err = splt_su_append_str(&word_with_tags, track_str, NULL);
           if (err != SPLT_OK) { goto error; }
           break;
@@ -812,17 +933,25 @@ int splt_tu_set_tags_in_tags(splt_state *state, int current_split)
 
   int replace_tags_in_tags = splt_o_get_int_option(state, SPLT_OPT_REPLACE_TAGS_IN_TAGS);
 
-  char *t = splt_tu_get_replaced_with_tags(tags->title, tags, track, &err, replace_tags_in_tags);
+  splt_tags *original_tags = splt_tu_get_original_tags_tags(state);
+
+  char *t = splt_tu_get_replaced_with_tags(tags->title, tags, original_tags,
+      track, &err, replace_tags_in_tags, current_split, state);
   if (err != SPLT_OK) { return err; }
-  char *y = splt_tu_get_replaced_with_tags(tags->year, tags, track, &err, replace_tags_in_tags);
+  char *y = splt_tu_get_replaced_with_tags(tags->year, tags, original_tags, track, &err, replace_tags_in_tags, 
+      current_split, state);
   if (err != SPLT_OK) { return err; }
-  char *a = splt_tu_get_replaced_with_tags(tags->artist, tags, track, &err, replace_tags_in_tags);
+  char *a = splt_tu_get_replaced_with_tags(tags->artist, tags, original_tags, track, &err, replace_tags_in_tags, 
+      current_split, state);
   if (err != SPLT_OK) { return err; }
-  char *al = splt_tu_get_replaced_with_tags(tags->album, tags, track, &err, replace_tags_in_tags);
+  char *al = splt_tu_get_replaced_with_tags(tags->album, tags, original_tags, track, &err, replace_tags_in_tags, 
+      current_split, state);
   if (err != SPLT_OK) { return err; }
-  char *c = splt_tu_get_replaced_with_tags(tags->comment, tags, track, &err, replace_tags_in_tags);
+  char *c = splt_tu_get_replaced_with_tags(tags->comment, tags, original_tags, track, &err, replace_tags_in_tags,
+      current_split, state);
   if (err != SPLT_OK) { return err; }
-  char *g = splt_tu_get_replaced_with_tags(tags->genre, tags, track, &err, replace_tags_in_tags);
+  char *g = splt_tu_get_replaced_with_tags(tags->genre, tags, original_tags, track, &err, replace_tags_in_tags,
+      current_split, state);
   if (err != SPLT_OK) { return err; }
 
   splt_su_free_replace(&cur_tags->title, t);
