@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2010 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2011 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -30,13 +30,13 @@
  *
  *********************************************************/
 
+/*! \file
+
+  Utilities that are needed when we want to talk to win32.
+ */
 #ifdef __WIN32__
 
 #include "splt.h"
-#include "win32.h"
-
-static wchar_t *splt_w32_encoding_to_utf16(UINT encoding, const char *source);
-static char *splt_w32_utf16_to_encoding(UINT encoding, const wchar_t *source);
 
 int scandir(const char *dir, struct dirent ***namelist,
 		int(*filter)(const struct dirent *),
@@ -250,11 +250,46 @@ int walphasort(const struct _wdirent **a, const struct _wdirent **b)
   return ret;
 }
 
+static wchar_t *splt_w32_encoding_to_utf16(UINT encoding, const char *source)
+{
+  wchar_t *dest = NULL;
+
+  int converted_size = MultiByteToWideChar(encoding, 0, source, -1, NULL, 0);
+  if (converted_size > 0)
+  {
+    dest = malloc(sizeof(wchar_t) * converted_size);
+    if (dest)
+    {
+      MultiByteToWideChar(encoding, 0, source, -1, dest, converted_size);
+    }
+  }
+
+  return dest;
+}
+
 wchar_t *splt_w32_utf8_to_utf16(const char *source)
 {
   return splt_w32_encoding_to_utf16(CP_UTF8, source);
 }
 
+
+static char *splt_w32_utf16_to_encoding(UINT encoding, const wchar_t *source)
+{
+  char *dest = NULL;
+
+  int converted_size = WideCharToMultiByte(encoding, 0, source, -1, NULL, 0, NULL, NULL);
+  if (converted_size > 0)
+  {
+    dest = malloc(sizeof(char *) * converted_size);
+    if (dest)
+    {
+      WideCharToMultiByte(encoding, 0, source, -1, dest, converted_size, NULL, NULL);
+    }
+  }
+
+  return dest;
+}
+
 char *splt_w32_utf16_to_utf8(const wchar_t *source)
 {
   return splt_w32_utf16_to_encoding(CP_UTF8, source);
@@ -307,39 +342,5 @@ int splt_w32_str_is_drive_root_directory(const char *str)
     splt_w32_str_starts_with_drive_root_directory(str);
 }
 
-static wchar_t *splt_w32_encoding_to_utf16(UINT encoding, const char *source)
-{
-  wchar_t *dest = NULL;
-
-  int converted_size = MultiByteToWideChar(encoding, 0, source, -1, NULL, 0);
-  if (converted_size > 0)
-  {
-    dest = malloc(sizeof(wchar_t) * converted_size);
-    if (dest)
-    {
-      MultiByteToWideChar(encoding, 0, source, -1, dest, converted_size);
-    }
-  }
-
-  return dest;
-}
-
-static char *splt_w32_utf16_to_encoding(UINT encoding, const wchar_t *source)
-{
-  char *dest = NULL;
-
-  int converted_size = WideCharToMultiByte(encoding, 0, source, -1, NULL, 0, NULL, NULL);
-  if (converted_size > 0)
-  {
-    dest = malloc(sizeof(char *) * converted_size);
-    if (dest)
-    {
-      WideCharToMultiByte(encoding, 0, source, -1, dest, converted_size, NULL, NULL);
-    }
-  }
-
-  return dest;
-}
-
 #endif
 
